# coding: utf-8
"""
helpers shared by the various QA tools
"""
from __future__ import print_function
from __future__ import unicode_literals
import sys


RED = "\033[1;31m"
GREEN = "\033[1;32m"
YELLOW = "\033[1;33m"
YELLOW_LIGHT = "\033[33m"
CLEAR = "\033[0;m"


def colorized(text, color):
    return '\n'.join(
        map(lambda line: color + line + CLEAR, text.split('\n')))


def reset(text):
    return colorized(text, CLEAR)


def green(text):
    return colorized(text, GREEN)


def yellow(text):
    return colorized(text, YELLOW)


def red(text):
    return colorized(text, RED)


def yellow_light(text):
    return colorized(text, YELLOW_LIGHT)


def print_flush(msg):
    if sys.version_info[0] == 3:
        print(msg, flush=True)
    else:
        print(msg)
        sys.stdout.flush()


fail_msg = red("FAIL")
success_msg = green("Success")



# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import sys
from . import scripts
from . import travis
from . import z0bug_odoo_lib
z0bugodoo = z0bug_odoo_lib.Z0bugOdoo()
try:
    import odoo.release as release
    from . import test_common
except ImportError:
    try:
        import openerp.release as release
        from . import test_common
    except ImportError:
        release = ''

__version__ = '1.0.12'
if eval(os.environ.get('TRAVIS_DEBUG_MODE', '0')) > 2:
    print('DEBUG: z0bug_odoo %s' % __version__)
    print('DEBUG: z0bug_odoo.sys.path=%s' % sys.path)
    if release:
        print('DEBUG: Odoo version detected: %s' % release.version)
    else:
        print('DEBUG: No Odoo environment found!')


# -*- coding: utf-8 -*-
# Copyright (C) 2018-2019 SHS-AV s.r.l. (<http://www.zeroincombenze.org>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
from __future__ import print_function
from builtins import str
import sys

from .z0bug_odoo_lib import Z0bugOdoo
try:
    import odoo.release as release
except ImportError:
    try:
        import openerp.release as release
    except ImportError:
        release = ''
if release:
    if int(release.major_version.split('.')[0]) < 10:
        import openerp.tests.common as test_common
        from openerp.modules.module import get_module_resource     # noqa: F401
        from openerp import workflow                               # noqa: F401
    else:
        import odoo.tests.common as test_common
        from odoo.modules.module import get_module_resource        # noqa: F401
else:
    print('No Odoo environment found!')
    sys.exit(0)

__version__ = '1.0.12'


class Z0bugBaseCase(test_common.BaseCase):

    def pool_env(self, model):
        """Return model pool_environment"""
        if int(release.major_version.split('.')[0]) < 8:
            return self.registry(model)
        return self.env[model]

    def create_id(self, model, values):
        """Create a new record for test"""
        if int(release.major_version.split('.')[0]) < 8:
            return self.registry(model).create(self.cr,
                                               self.uid,
                                               values)
        return self.env[model].create(values).id

    def create_rec(self, model, values):
        """Create a new record for test"""
        if int(release.major_version.split('.')[0]) < 8:
            return self.registry(model).browse(
                self.cr,
                self.uid,
                self.registry(model).create(self.cr,
                                            self.uid,
                                            values))
        return self.env[model].create(values)

    def write_rec(self, model, id, values):
        """Write existent record"""
        if int(release.major_version.split('.')[0]) < 8:
            return self.registry(model).write(self.cr, self.uid, [id], values)
        return self.env[model].browse(id).write(values)

    def write_ref(self, xref, values):
        """Browse and write existent record"""
        return self.browse_ref(xref).write(values)

    def browse_rec(self, model, id):
        if int(release.major_version.split('.')[0]) < 8:
            return self.registry(model).browse(self.cr, self.uid, id)
        return self.env[model].browse(id)

    def search_rec(self, model, args):
        """Search records - Syntax search(model, *args)
        Warning! Do not use with Odoo 7.0: result may fails!"""
        if int(release.major_version.split('.')[0]) < 8:
            ir_model = self.registry(model)
            return ir_model.browse(self.cr, self.uid,
                                   ir_model.search(self.cr, self.uid, args))
        return self.env[model].search(args)

    def ref_id(self, xref):
        """Return reference id. Do not crash if xref does not exist"""
        if int(release.major_version.split('.')[0]) < 8:
            if xref.startswith('base.state_it_'):
                # 7.0 compatibility
                try:
                    return self.ref('l10n_it_base.it_%s' % xref[14:].upper())
                except BaseException:
                    pass
            try:
                return self.ref(xref)
            except BaseException:
                return False
        try:
            return self.env.ref(xref).id
        except BaseException:
            return False

    def ref_rec(self, xref):
        """Return reference record. Do not crash if xref does not exist"""
        if int(release.major_version.split('.')[0]) < 8:
            if xref.startswith('base.state_it_'):
                # 7.0 compatibility
                try:
                    return self.browse_ref(
                        'l10n_it_base.it_%s' % xref[14:].upper())
                except BaseException:
                    pass
            try:
                return self.browse_ref(xref)
            except BaseException:
                return False
        try:
            return self.env.ref(xref)
        except BaseException:
            return False

    def settle_fields(self, model, vals, how_id=None):
        for name in vals.copy():
            if name == 'id':
                if how_id == 'del':
                    del vals[name]
                    continue
                elif how_id == 'keep':
                    continue
            field = self.search_rec('ir.model.fields',
                                    [('model', '=', model),
                                     ('name', '=', name)])
            if isinstance(field, (list, tuple)) and len(field):
                field = field[0]
            if not field:
                del vals[name]
            elif model == 'res.partner' and name in ('lang', 'state_id'):
                # Tests w/o italian language
                del vals[name]
            elif (field.ttype == 'many2one' and
                  isinstance(vals[name], str) and
                  len(vals[name].split('.')) == 2):
                vals[name] = self.ref_id(vals[name])
        return vals

    def get_ref_value(self, model, xref):
        if not hasattr(self, 'Z0bugOdoo'):
            self.Z0bugOdoo = Z0bugOdoo()
        return self.settle_fields(
            model, self.Z0bugOdoo.get_test_values(model, xref),
            how_id='keep')

    def build_model_data(self, model, xrefs):
        if not isinstance(xrefs, (list, tuple)):
            xrefs = [xrefs]
        # import pdb
        # pdb.set_trace()
        for xref in sorted(xrefs):
            vals = self.get_ref_value(model, xref)
            if not vals:
                pass
            elif 'id' in vals:
                xids = xref.split('.')
                if len(xids) == 2:
                    try:
                        id = self.ref_id(xref)
                    except BaseException:
                        id = None
                elif vals['id']:
                    id = vals['id']
                else:
                    id = None
                del vals['id']
                if id:
                    self.write_rec(model, id, vals)
                else:
                    id = self.create_id(model, vals)
                    if len(xids) == 2:
                        vals = {
                            'module': xids[0],
                            'model': model,
                            'name': xids[1],
                            'res_id': id,
                        }
                        self.create_rec('ir.model.data', vals)
            else:
                raise KeyError('Invalid xref %s for model %s!' % (xref, model))

    def set_test_company(self, xref=None):
        """Set company to test"""
        if not xref:
            for xref1, model in (('z0bug.partner_mycompany', 'res.partner'),
                                 ('z0bug.mycompany', 'res.company')):
                self.build_model_data(model, xref1)
            xref = 'z0bug.mycompany'
        xref_id = self.ref_id(xref)
        if not xref_id:
            # In case of trouble, test company is the main company
            xref_id = self.ref_id('base.main_company')
        # There are two separate write because "company_id" assignment fails if
        # company_id is not in "company_ids" at the time of the write
        if int(release.major_version.split('.')[0]) < 8:
            self.registry('res.users').write(
                self.cr, self.uid, [self.uid], {'company_ids': [(4, xref_id)]})
            self.registry('res.users').write(
                self.cr, self.uid, [self.uid], {'company_id': xref_id})
        else:
            self.env.user.write({'company_ids': [(4, xref_id)]})
            self.env.user.write({'company_id': xref_id})
        return xref_id

    def bind_fields(self, model, vals, company_id,
                    parent_id=None, parent_model=None):
        """TODO: write implementation"""
        return vals

    def write_diff(self, model, id, vals):
        """TODO: write implementation"""
        return self.write_rec(model, id, vals)

    def get_domain_field(self, model, vals, company_id,
                         parent_id=None, parent_name=None):
        """TODO: write implementation"""
        return False

    def add_xref(self, xref, model, xid):
        xrefs = xref.split('.')
        if len(xrefs) == 2:
            vals = {
                'module': xrefs[0],
                'model': model,
                'name': xrefs[1],
                'res_id': id,
            }
            return self.create_rec('ir.model.data', vals)
        return False

    def store_xref(self, xref, model, company_id,
                   parent_id=None, parent_model=None, force=None):
        if parent_id and parent_model:
            xid = False
        else:
            xid = self.ref_id(xref)
        if not xid or force:
            vals = self.Z0bugOdoo.get_test_values(model, xref)
            vals, parent_name = self.bind_fields(
                model, vals, company_id,
                parent_id=parent_id, parent_model=parent_model)
            if xid:
                self.write_diff(model, xid, vals)
            else:
                if vals.get('id') and isinstance(vals['id'], int):
                    xid = vals['id']
                else:
                    xid = self.get_domain_field(model, vals, company_id,
                                                parent_id=parent_id,
                                                parent_name=parent_name)
                if xid:
                    self.write_diff(model, xid, vals)
                else:
                    if 'id' in vals:
                        del vals['id']
                    xid = self.create_id(vals)
                    self.ctr_rec_new += 1
                if not parent_id or not parent_model:
                    self.add_xref(xref, model, xid)
        return xid

    def make_model_data(
            self, model, model2=None, company_id=None, xref=None, force=None):
        """Create a full table with demo data"""
        if model == 'res.country.state':
            xrefs = self.Z0bugOdoo.get_test_xrefs(model)
            for xref in sorted(xrefs):
                self.store_xref(xref, model, company_id, force=force)
        raise KeyError('Unsupported model%s to create!' % model)


class TransactionCase(test_common.TransactionCase, Z0bugBaseCase):

    def setUp(self):
        Z0bugBaseCase.setUp(self)

    def pool_env(self, model):
        return Z0bugBaseCase.pool_env(self, model)

    def create_id(self, model, values):
        return Z0bugBaseCase.create_id(self, model, values)

    def create_rec(self, model, values):
        return Z0bugBaseCase.create_rec(self, model, values)

    def write_rec(self, model, id, values):
        return Z0bugBaseCase.write_rec(self, model, id, values)

    def write_ref(self, model, id, values):
        return Z0bugBaseCase.write_ref(self, model, id, values)

    def browse_rec(self, model, id):
        return Z0bugBaseCase.browse_rec(self, model, id)

    def search_rec(self, model, args):
        return Z0bugBaseCase.search_rec(self, model, args)

    def ref_id(self, xref):
        return Z0bugBaseCase.ref_id(self, xref)

    def ref_rec(self, xref):
        return Z0bugBaseCase.ref_rec(self, xref)

    def settle_fields(self, model, vals, how_id=None):
        return Z0bugBaseCase.settle_fields(self, model, vals, how_id)

    def get_ref_value(self, model, xref):
        return Z0bugBaseCase.get_ref_value(self, model, xref)

    def build_model_data(self, model, xrefs):
        return Z0bugBaseCase.build_model_data(self, model, xrefs)

    def set_test_company(self, xref=None):
        return Z0bugBaseCase.set_test_company(self, xref)


class SingleTransactionCase(test_common.SingleTransactionCase, Z0bugBaseCase):

    def pool_env(self, model):
        return Z0bugBaseCase.pool_env(self, model)

    def create_id(self, model, values):
        return Z0bugBaseCase.create_id(self, model, values)

    def create_rec(self, model, values):
        return Z0bugBaseCase.create_rec(self, model, values)

    def write_rec(self, model, id, values):
        return Z0bugBaseCase.write_rec(self, model, id, values)

    def write_ref(self, model, id, values):
        return Z0bugBaseCase.write_ref(self, model, id, values)

    def browse_rec(self, model, id):
        return Z0bugBaseCase.browse_rec(self, model, id)

    def settle_fields(self, model, vals, how_id=None):
        return Z0bugBaseCase.settle_fields(self, model, vals, how_id)

    def get_ref_value(self, model, xref):
        return Z0bugBaseCase.get_ref_value(self, model, xref)

    def build_model_data(self, model, xrefs):
        return Z0bugBaseCase.build_model_data(self, model, xrefs)

    def set_test_company(self, xref=None):
        return Z0bugBaseCase.set_test_company(self, xref)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import ast
import os
import re
import sys
import inspect

import click
import pylint.lint

if sys.version_info[0] == 2:
    import travis_helpers
    from getaddons import get_modules_changed, is_module
    from git_run import GitRun
else:
    from . import travis_helpers
    from .getaddons import get_modules_changed, is_module
    from .git_run import GitRun

try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser


CLICK_DIR = click.Path(exists=True, dir_okay=True, resolve_path=True)


def get_extra_params(odoo_version):
    """Get extra pylint params by odoo version
    Transform a pseudo-pylint-conf to params,
    to overwrite base-pylint-conf values.
    Use a seudo-inherit of configuration file.
    To avoid having 2 config files (stable and pr-conf) by each odoo-version
    Example:

        pylint_master.conf
        pylint_master_pr.conf
        pylint_90.conf
        pylint_90_pr.conf
        pylint_80.conf
        pylint_80_pr.conf
        pylint_70.conf
        pylint_70_pr.conf
        pylint_61.conf
        pylint_61_pr.conf
        ... and new future versions.

    If you need to add new conventions in all versions,
    you will need to change all pr files or stable files.


    With this method you can use:

        pylint_lastest.conf
        pylint_lastest_pr.conf
        pylint_disabling_70.conf <- Overwrite params of pylint_lastest*.conf
        pylint_disabling_61.conf <- Overwrite params of pylint_lastest*.conf

    If you need to add new conventions in all versions, you will only need to
    change pylint_lastest_pr.conf or pylint_lastest.conf, similar to inherit.

    :param version: String to specify an Odoo's name or versio
    :return: List of extra pylint params
    """
    is_version_number = re.match(r'\d+\.\d+', odoo_version)
    beta_msgs = get_beta_msgs()
    leverage_msgs = get_leverage_msgs()
    versioned_msgs = get_versioned_msgs(odoo_version)

    extra_params_cmd = [
        '--sys-paths', os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'pylint_deprecated_modules'),
        '--extra-params', '--load-plugins=pylint_odoo']

    extra_params = list(extra_params_cmd)
    if is_version_number:
        extra_params.extend([
            '--extra-params', '--valid_odoo_versions=%s' % odoo_version])
    for beta_msg in beta_msgs:
        extra_params.extend(['--msgs-no-count', beta_msg,
                             '--extra-params', '--enable=%s' % beta_msg])
    # [antoniov: 2018-09-12]
    for leverage_msg in leverage_msgs:
        extra_params.extend(['--msgs-no-count', leverage_msg,
                             '--extra-params', '--disable=%s' % leverage_msg])
    # [antoniov: 2020-07-20]
    for versioned_msg in versioned_msgs:
        extra_params.extend(['--extra-params', '--disable=%s' % versioned_msg])
    return extra_params


def get_versioned_msgs(odoo_version):
    """[antoniov: 2020-07-20] enable or disable some check
     depending on Odoo version
    :return: List of strings with beta message names"""
    odoo_version = odoo_version.replace('.', '')
    specific_cfg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'cfg/travis_run_pylint_exclude_%s.cfg' % (odoo_version))
    if not os.path.isfile(specific_cfg):
        return []
    config = ConfigParser.ConfigParser()
    config.readfp(open(specific_cfg))
    params = []
    for section in config.sections():
        for option, value in config.items(section):
            params.extend(['--' + option, value.strip().replace('\n', '')])
    return params


def get_leverage_msgs():
    """[antoniov: 2018-09-12] Based on LINT_CHECK_LEVEL, disable some check
    :return: List of strings with beta message names"""
    exclude_level = os.environ.get('LINT_CHECK_LEVEL', '')
    specific_cfg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'cfg/travis_run_pylint_exclude_%s.cfg' % exclude_level)
    if not os.path.isfile(specific_cfg):
        return []
    config = ConfigParser.ConfigParser()
    config.readfp(open(specific_cfg))
    params = []
    for section in config.sections():
        for option, value in config.items(section):
            params.extend(['--' + option, value.strip().replace('\n', '')])
    return params


def get_beta_msgs():
    """Get beta msgs from beta.cfg file
    :return: List of strings with beta message names"""
    beta_cfg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'cfg/travis_run_pylint_beta.cfg')
    if not os.path.isfile(beta_cfg):
        return []
    config = ConfigParser.ConfigParser()
    config.readfp(open(beta_cfg))
    return [
        msg.strip().replace('\n', '')
        for msg in config.get('MESSAGES CONTROL', 'enable').split(',')
        if msg.strip()]


def get_modules_cmd(dir):
    modules_cmd = []
    include_lint = os.environ.get('INCLUDE_LINT')
    if include_lint:
        for path in include_lint.split(' '):
            modules_cmd.extend(['--path', path])
    else:
        modules_cmd.extend(["--path", dir])
    return modules_cmd


def version_validate(version, dir):
    if not version and dir:
        repo_path = os.path.join(dir, '.git')
        branch_name = GitRun(repo_path).get_branch_name()
        version = (branch_name.replace('_', '-').split('-')[:1]
                   if branch_name else False)
        version = version[0] if version else None
    if not version:
        print(travis_helpers.yellow(
            'Undefined environment variable'
            ' `VERSION`.\nSet `VERSION` for '
            'compatibility with guidelines by version.'))
    return version


def get_branch_base():
    branch_base = os.environ.get('TRAVIS_BRANCH') or os.environ.get('VERSION')
    if branch_base != 'HEAD':
        branch_base = 'origin/' + (branch_base and branch_base or '')
    return branch_base


def pylint_run(is_pr, version, dir):
    # Look for an environment variable
    # whose value is the name of a proper configuration file for pylint
    # (this file will then be expected to be found in the 'cfg/' folder).
    # If such an environment variable is not found,
    # it defaults to the standard configuration file.
    pylint_config_file = os.environ.get(
        'PYLINT_CONFIG_FILE', 'travis_run_pylint.cfg')
    pylint_rcfile = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), 'cfg', pylint_config_file)
    pylint_rcfile_pr = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'cfg', "travis_run_pylint_pr.cfg")
    odoo_version = version_validate(version, dir)
    modules_cmd = get_modules_cmd(dir)
    beta_msgs = get_beta_msgs()
    branch_base = get_branch_base()
    extra_params_cmd = get_extra_params(odoo_version)
    extra_info = "extra_params_cmd %s " % extra_params_cmd
    print(extra_info)
    conf = ["--config-file=%s" % (pylint_rcfile)]
    cmd = conf + modules_cmd + extra_params_cmd

    real_errors = main(cmd, standalone_mode=False)
    res = dict(
        (key, value) for key, value in (real_errors.get(
            'by_msg') or {}).items() if key not in beta_msgs)
    count_errors = get_count_fails(real_errors, list(beta_msgs))
    count_info = "count_errors %s" % count_errors
    print(count_info)
    if is_pr:
        print(travis_helpers.green(
            'Starting lint check only for modules changed'))
        modules_changed = get_modules_changed(dir, branch_base)
        if not modules_changed:
            print(travis_helpers.green(
                'There are not modules changed from '
                '"git --git-dir=%s diff ..%s"' % (dir, branch_base)))
            return res
        modules_changed_cmd = []
        for module_changed in modules_changed:
            modules_changed_cmd.extend(['--path', module_changed])
        conf = ["--config-file=%s" % (pylint_rcfile_pr)]
        cmd = conf + modules_changed_cmd + extra_params_cmd
        pr_real_errors = main(cmd, standalone_mode=False)
        pr_stats = dict(
            (key, value) for key, value in (pr_real_errors.get(
                'by_msg') or {}).items() if key not in beta_msgs)
        if pr_stats:
            pr_errors = get_count_fails(pr_real_errors, list(beta_msgs))
            print(travis_helpers.yellow(
                "Found %s errors in modules changed." % (pr_errors)))
            if pr_errors < 0:
                res = pr_stats
            else:
                new_dict = {}
                for val in res:
                    new_dict[val] = (new_dict.get(val, 0) + res[val])
                for val in pr_stats:
                    new_dict[val] = (new_dict.get(val, 0) + pr_stats[val])
                res = new_dict
    return res


def get_count_fails(linter_stats, msgs_no_count=None):
    """Verify the dictionary statistics to get number of errors.
    :param linter_stats: Dict of type pylint.lint.Run().linter.stats
    :param no_count: List of messages that will not add to the failure count.
    :return: Integer with quantity of fails found.
    """
    return sum([
        linter_stats['by_msg'][msg]
        for msg in (linter_stats.get('by_msg') or {})
        if msg not in msgs_no_count])


def is_installable_module(path):
    """return False if the path doesn't contain an installable odoo module,
    otherwise the full path to the module's manifest"""
    manifest_path = is_module(path)
    if manifest_path:
        manifest = ast.literal_eval(open(manifest_path).read())
        if manifest.get('installable', True):
            return manifest_path
    return False


def get_subpaths(paths, depth=1):
    """Get list of subdirectories
    if `__init__.py` file doesn't exists in root path, then
    get subdirectories.
    Why? More info here:
        https://www.mail-archive.com/code-quality@python.org/msg00294.html
    :param paths: List of paths
    :param depth: How many folders can be opened in deep to find a module.
    :return: Return list of paths with subdirectories.
    """
    subpaths = []
    for path in paths:
        if depth < 0:
            continue
        if not os.path.isfile(os.path.join(path, '__init__.py')):
            new_subpaths = [os.path.join(path, item)
                            for item in os.listdir(path)
                            if os.path.isdir(os.path.join(path, item))]
            if new_subpaths:
                subpaths.extend(get_subpaths(new_subpaths, depth-1))
        else:
            if is_installable_module(path):
                subpaths.append(path)
    return subpaths


def run_pylint(paths, cfg, beta_msgs=None, sys_paths=None, extra_params=None):
    """Execute pylint command from original python library
    :param paths: List of paths of python modules to check with pylint
    :param cfg: String name of pylint configuration file
    :param sys_paths: List of paths to append to sys path
    :param extra_params: List of extra parameters to append
        in pylint command
    :return: Dict with python linter stats
    """
    if sys_paths is None:
        sys_paths = []
    if extra_params is None:
        extra_params = []
    sys.path.extend(sys_paths)
    cmd = ['--rcfile=' + cfg]
    cmd.extend(extra_params)
    subpaths = get_subpaths(paths)
    if not subpaths:
        raise UserWarning("Python modules not found in paths %s" % (paths))
    exclude = os.environ.get('EXCLUDE', '').split(',')
    subpaths = [path for path in subpaths
                if os.path.basename(path) not in exclude]
    if not subpaths:
        return {'error': 0}
    cmd.extend(subpaths)
    if 'do_exit' in inspect.getargspec(pylint.lint.Run.__init__)[0]:
        # pylint has renamed this keyword argument
        pylint_res = pylint.lint.Run(cmd, do_exit=False)
    else:
        pylint_res = pylint.lint.Run(cmd, exit=False)
    return pylint_res.linter.stats


@click.command()
@click.option('paths', '--path', envvar='TRAVIS_BUILD_DIR',
              multiple=True, type=CLICK_DIR, required=True,
              default=[os.getcwd()],
              help="Addons paths to check pylint")
@click.option('--config-file', '-c',
              type=click.File('r', lazy=True), required=True,
              help="Pylint config file")
@click.option('--sys-paths', '-sys-path', envvar='PYTHONPATH',
              multiple=True, type=CLICK_DIR,
              help="Additional paths to append in sys path.")
@click.option('--extra-params', '-extra-param', multiple=True,
              help="Extra pylint params to append "
                   "in pylint command")
@click.option('--msgs-no-count', '-msgs-no-count', multiple=True,
              help="List of messages that will not add to the failure count.")
def main(paths, config_file, msgs_no_count=None,
         sys_paths=None, extra_params=None):
    """Script to run pylint command with additional params
    to check fails of odoo modules.
    If expected errors is equal to count fails found then
    this program exits with zero, otherwise exits with counted fails"""
    try:
        stats = run_pylint(
            list(paths), config_file.name,
            sys_paths=sys_paths,
            extra_params=extra_params)
    except UserWarning:
        stats = {'error': -1}
    return stats


if __name__ == '__main__':
    try:
        exit(main(standalone_mode=False))
    except click.ClickException as e:
        e.show()
        exit(e.exit_code)


# coding: utf-8

import base64
import os
import json
import requests


class ApiException(Exception):
    pass


class Request(object):

    def __init__(self):
        self.session = requests.Session()

    def _check(self):
        if not self._token:
            raise ApiException("WARNING! Token not defined exiting early.")
        self.session.headers.update({
            'Accept': 'application/json',
            'User-Agent': 'mqt',
            'Authorization': 'Token %s' % self._token
        })
        self._request(self.host)

    def _request(self, url, payload=None, is_json=True, patch=False):
        try:
            if not payload and not patch:
                response = self.session.get(url)
            elif patch:
                response = self.session.patch(url, data=payload)
            else:
                response = self.session.post(url, data=payload)
            response.raise_for_status()
        except requests.RequestException as error:
            raise ApiException(str(error))
        return response.json() if is_json else response


class GitHubApi(Request):

    def __init__(self):
        super(GitHubApi, self).__init__()
        self._token = os.environ.get("GITHUB_TOKEN")
        self.host = "https://api.github.com"
        self._owner, self._repo = os.environ.get("TRAVIS_REPO_SLUG").split('/')

    def create_pull_request(self, data):
        pull = self._request(self.host + '/repos/%s/%s/pulls' %
                             (self._owner, self._repo), json.dumps(data))
        return pull

    def create_commit(self, message, branch, files):
        tree = []
        info_branch = self._request(
            self.host + '/repos/%s/%s/git/refs/heads/%s' %
            (self._owner, self._repo, branch))
        branch_commit = self._request(
            self.host + '/repos/%s/%s/git/commits/%s' %
            (self._owner, self._repo, info_branch['object']['sha']))
        for item in files:
            with open(item) as f_po:
                blob_data = json.dumps({
                    'content': base64.b64encode(f_po.read()),
                    'encoding': 'base64'
                })
                blob_sha = self._request(
                    self.host + '/repos/%s/%s/git/blobs' %
                    (self._owner, self._repo), blob_data)
                tree.append({
                    'path': item,
                    'mode': '100644',
                    'type': 'blob',
                    'sha': blob_sha['sha']
                })
        tree_data = json.dumps({
            'base_tree': branch_commit['tree']['sha'],
            'tree': tree
        })
        info_tree = self._request(self.host + '/repos/%s/%s/git/trees' %
                                  (self._owner, self._repo), tree_data)
        commit_data = json.dumps({
            'message': message,
            'tree': info_tree['sha'],
            'parents': [branch_commit['sha']]
        })
        info_commit = self._request(self.host + '/repos/%s/%s/git/commits' %
                                    (self._owner, self._repo), commit_data)
        update_branch = self._request(
            self.host + '/repos/%s/%s/git/refs/heads/%s' %
            (self._owner, self._repo, branch),
            json.dumps({'sha': info_commit['sha']}),
            patch=True)
        return info_commit['sha'] == update_branch['object']['sha']



#!/usr/bin/env python
"""
Usage: get_test_dependencies.py -r|-m|-d|mod|dep -1 path_list [module_list]

If 1st switch is 'mod' or -m, return module list from path_list (comma separated) 
If 1st switch is 'dep' or -d, return dependencies list from path_list
for specific module_list (comma separated) if supplied or
module_list is generated from path_list.
if -1 is supplied return just 1 level depth dependecies (no recurse)
If 1st swicth is -r, return module which dependes from module_list
"""
from __future__ import print_function

import sys
import os
if sys.version_info[0] == 2:
    from getaddons import (get_modules, get_dependents,
                           get_dependencies, get_modules_info)
    from test_server import get_test_dependencies
else:
    from .getaddons import (get_modules, get_dependents,
                            get_dependencies, get_modules_info)
    from .test_server import get_test_dependencies


__version__ = '1.0.12'


def get_module_list(paths):
    res = []
    for path in paths.split(','):
        r = get_modules(os.path.expanduser(path))
        for m in r:
            if m not in res:
                res.append(m)
    return res


def get_dep_of_module(addons_path, addons_list):
    if not addons_list:
        return ['base']
    else:
        modules = {}
        for path in addons_path.split(','):
            modules.update(get_modules_info(path))
        dependencies = set()
        for module in addons_list:
            dependencies |= get_dependencies(modules, module)
        return list(dependencies - set(addons_list))


def main(argv=None):
    if argv is None:
        argv = sys.argv
    argv = argv[1:]
    if not argv:
        print(__doc__)
        return 1

    list_type = False
    paths = ''
    module_list = None
    not_recurse = False

    params = []
    while argv:
        if argv[0].startswith('-'):
            param = argv.pop(0)
            if param == '-h' or param == '--help':
                print(__doc__)
                return 1
            elif param == '-V' or param == '--version':
                print(__version__)
                return 1
            elif param == '-d':
                list_type = 'dep'
            elif param == '-m':
                list_type = 'mod'
            elif param == '-r':
                list_type = 'req'
            elif param == '-1':
                not_recurse = True
            else:
                raise Exception('Unknown parameter: %s' % param)
        else:
            params.append(argv.pop(0))

    if not list_type and len(params):
        list_type = params.pop(0)
    if len(params):
        paths = params.pop(0)
    if len(params):
        module_list = params.pop(0).split(',')

    if list_type == 'mod':
        print(','.join(get_module_list(paths)))
    elif list_type == 'dep':
        if not module_list:
            module_list = get_module_list(paths)
        if not_recurse:
            modules = get_modules_info(paths, depth=1)
            res = []
            for module in module_list:
                res += modules[module].get('depends', [])
        else:
            res = get_test_dependencies(paths, module_list)
        print(','.join(res))
    elif list_type == 'req':
        if not module_list:
            module_list = get_module_list(paths)
        modules = get_modules_info(paths, depth=1)
        res = []
        for module in module_list:
            res += get_dependents(modules, module)
        print(','.join(res))
    else:
        print(__doc__)
        return 1
    return 0


if __name__ == "__main__":
    sys.exit(main())